import { MetricsPanelCtrl } from 'app/plugins/sdk';
import _ from 'lodash';


export class Apedex extends MetricsPanelCtrl {
  constructor($scope, $injector) {
    super($scope, $injector);
    _.defaultsDeep(this.panel, panelDefaults);

    this.events.on('data-received', this.onDataReceived.bind(this));
    this.events.on('init-edit-mode', this.onInitEditMode.bind(this));

    this.data = [{ label: 'No data', value: NaN }]
  }

  onDataReceived(data=[]) {
    this.data = data.map(d => (
      d.datapoints.reduce((result, [val, data]) => (
        zonesPredicates(this.panel.satisfiedZone).reduce((acc, f) => f(acc, val), result)
      ), { sz: 0, tz: 0, total: d.datapoints.length })
    ))
    .map(data => ({
      value: calcApedex(data.sz, data.tz, data.total)
    }))
    .map(data => ({
      value: data.value,
      label: labelPredicates().reduce((acc, f) => f(acc, data.value), { text: 'Excellent', color: '#258e2d' })
    }))

    this.render()
  }

  onInitEditMode() {
    this.addEditorTab('Options', 'public/plugins/apdex/editor.html', 2);
  }

  link(scope, elem, attrs, ctrl) {
    this.events.on('render', () => {
      elem.find('.panel-container').css('background-color', this.data.reduce((_, d) => (d.label.color), '#000'))
      elem.find('.panel-title').css('color', '#FFF')
    })
  }
}

Apedex.templateUrl = 'module.html';

const zonesPredicates = (T) => [
  (acc, val) => val <= T
    ? Object.assign(acc, { sz: acc.sz + 1 })
    : acc,
  (acc, val) => val > T && val <= T * 4
    ? Object.assign(acc, { tz: acc.tz + 1 })
    : acc,
]

const labelPredicates = () => [
  (before, val) => val <= 0.93 ? { text: 'Good', color: '#89ca0b' } : before,
  (before, val) => val <= 0.84 ? { text: 'Fair', color: '#ffba10' } : before,
  (before, val) => val <= 0.69 ? { text: 'Poor', color: '#f76522' } : before,
  (before, val) => val <= 0.49 ? { text: 'Unacceptable', color: '#cf2122' } : before,
]

const calcApedex = (sz, tz, total) => (
  Number(((sz + (tz / 2)) / total).toPrecision(2))
)

const log = msg => x => (console.log(msg, x), x)

const panelDefaults = {
  labelSettings: {
    fontSize: '30px',
    fontWeight: 'bold',
    color: 'white',
    textTransform: 'uppercase'
  },
  valueSettings: {
    fontSize: '20px',
    fontWeight: 'normal',
    color: 'white',
    textTransform: 'normal'
  },
  satisfiedZone: 5
};

