import {QueryCtrl} from 'app/plugins/sdk';
import './css/query-editor.css!'
import * as c from './constants';

export class OpMonDatasourceQueryCtrl extends QueryCtrl {

  constructor($scope, $injector, uiSegmentSrv)  {
    super($scope, $injector);

    this.scope = $scope;
    this.uiSegmentSrv     = uiSegmentSrv;
    this.target.host      = this.target.host      || this.datasource.DEFAULT_HOST;
    this.target.hostgroup = this.target.hostgroup || this.datasource.DEFAULT_HOSTGROUP;
    this.target.service   = this.target.service   || this.datasource.DEFAULT_SERVICE;
    this.target.servicegroup = this.target.servicegroup || this.datasource.DEFAULT_SERVICEGROUP;
    this.target.serviceCatalog = this.target.serviceCatalog || this.datasource.DEFAULT_SERVICECATALOG;
    this.target.metric = this.target.metric || this.datasource.DEFAULT_METRIC;
    //this.target.timecut = this.target.timecut      || this.datasource.DEFAULT_TIMECUT;
    this.target.mode = (this.$scope.$root.mode) ? (this.$scope.$root.mode) : (this.target.mode || c.MODE_AVAIL);
    //this.target.groupby      = this.target.groupby      || 'none';
    //this.target.resultFormat = this.target.resultFormat      || this.datasource.DEFAULT_RESULT_FORMAT;
    this.target.objecttype = (this.$scope.$root.objecttype) ? this.$scope.$root.objecttype : (this.target.objecttype || this.datasource.DEFAULT_OBJECT_TYPE);
    this.target.extended_state = this.$scope.$root.extended_state ? this.$scope.$root.extended_state : this.target.extended_state || this.datasource.DEFAULT_EXTENDED_STATE;
    this.target.hardState = this.$scope.$root.hardState ? this.$scope.$root.hardState : this.target.hardState !== undefined ? this.target.hardState : this.datasource.DEFAULT_HARD_STATE;
    //this.target.fill      = this.target.fill      || 'fill';
    this.target.resultformat = (this.$scope.$root.resultformat) ? (this.$scope.$root.resultformat) : (this.target.resultformat || this.datasource.DEFAULT_RESULT_FORMAT);

    this.target.timecut = (this.$scope.$root.timecut) ? this.$scope.$root.timecut : (this.target.timecut || this.datasource.DEFAULT_TIMECUT);
    this.target.groupby = (this.$scope.$root.groupby) ? this.$scope.$root.groupby : (this.target.groupby || 'none');

    this.editorModes = [
      {value: 'avail',      text: 'Availability',        mode: c.MODE_AVAIL},
      {value: 'cp',       text: 'Capacity',     mode: c.MODE_CP},
      {value: 'system',       text: 'System',     mode: c.MODE_SYSTEM},
      {value: 'display_status',       text: 'Status',     mode: c.MODE_STATUS},
     // {value: 'avail_service', text: 'Availability - Service', mode: c.MODE_AVAIL_SERVICE},
 //     {value: 'itemid',    text: 'Item ID',     mode: c.MODE_ITEMID},
   //   {value: 'triggers',  text: 'Triggers',    mode: c.MODE_TRIGGERS}
    ];


    this.$scope.editorMode = {
      AVAIL: c.MODE_AVAIL,
      CP: c.MODE_CP,
      SYSTEM: c.MODE_SYSTEM,
      STATUS: c.MODE_STATUS
 //     ITEMID: c.MODE_ITEMID,
  //    TRIGGERS: c.MODE_TRIGGERS
    };


    /*if (this.$scope.$root.p) {
      this.$scope.$root.p.push(this.panelCtrl)
    } else {
      this.$scope.$root.p = [this.panelCtrl]
    }*/


    this.slaPropertyList = [
      {name: "Status", property: "status"},
      {name: "SLA", property: "sla"},
      {name: "OK time", property: "okTime"},
      {name: "Problem time", property: "problemTime"},
      {name: "Down time", property: "downtimeTime"}
    ];

    this.ackFilters = [
      {text: 'all triggers', value: 2},
      {text: 'unacknowledged', value: 0},
      {text: 'acknowledged', value: 1},
    ];

    this.resultFormats = [{ text: 'Time series', value: 'time_series' }, { text: 'Table', value: 'table' }];



  }

  /**
   * Switch query editor to specified mode.
   * Modes:
   *  0 - items
   *  1 - IT services
   *  2 - Text metrics
   */
  switchValue(name,value) {
    this.target[name] = value;
    this.$scope.$root[name] = value;
    if (name === 'mode' && value === c.MODE_CP && this.target.objecttype === 'Host') {
      this.target.objecttype = "Service";
    }
    console.log(name,value);
    //this.init();
    this.onChangeInternal();
  }

  getHost() {
    return this.datasource.metricFindData("host", this.target, true)
      .then(this.uiSegmentSrv.transformToSegments(false));
  }

  getHostGroup() {
    return this.datasource.metricFindData("hostgroup", this.target, true)
      .then(this.uiSegmentSrv.transformToSegments(false));
  }

  getService() {
    return this.datasource.metricFindData("service", this.target, true)
      .then(this.uiSegmentSrv.transformToSegments(false));
  }

  getServiceGroup() {
    return this.datasource.metricFindData("servicegroup", this.target, true)
      .then(this.uiSegmentSrv.transformToSegments(false));
  }

  getServiceCatalog() {
    return this.datasource.metricFindData("serviceCatalog", this.target, true)
      .then(this.uiSegmentSrv.transformToSegments(false));
  }


  getMetric() {
    return this.datasource.metricFindData("metric", this.target, true)
      .then(this.uiSegmentSrv.transformToSegments(false));
  }

  getTimecut() {
    return this.datasource.metricFindData("timecut", this.target, true)
      .then(this.uiSegmentSrv.transformToSegments(false));
  }

  onChangeInternal() {
    //this.$scope.$root.p.forEach((e) => {
    //  e.refresh()
    //})
    this.panelCtrl.refresh();
  }

  getCollapsedText() {
    if(this.target.metric == this.datasource.DEFAULT_METRIC &&
       this.target.host      == this.datasource.DEFAULT_HOST &&
       this.target.service   == this.datasource.DEFAULT_SERVICE) {
        return("click to edit query");
    }
    return(this.target.metric
           +': '+this.target.host
           +' - '+this.target.service
           );
  }
}

OpMonDatasourceQueryCtrl.templateUrl = 'partials/query.editor.html';
