
const pluck = (...ks) => x => ks.reduce((y, k) => y && y[k], x)

const getRangeFrom = pluck('range', 'from')
const getRangeTo = pluck('range', 'to')

export default class DataSource {
  constructor(instanceSettings, backendSrv) {
    this.id = instanceSettings.id;
    this.backendSrv = backendSrv;
    this.url = '/api/datasources/proxy/1/opmon/ws/oracle.php?endpoint=';
  }

  query(options) {
    const queries = options.targets
      .filter(t => !t.hide)
      .map(t => ({
        intervalMs: options.intervalMs,
        maxDataPoints: options.maxDataPoints,
        datasourceId: this.id,
        refId: t.refId,
        rawSql: t.rawSql,
        format: t.format
      }))

    return this.backendSrv.datasourceRequest({
      url: this.url + '/query',
      data: {
        from: (getRangeFrom(options) || '').valueOf().toString(),
        to: (getRangeTo(options) || '').valueOf().toString(),
        queries
      },
      method: 'POST'
    })
    .then(({ data }) => (
      Object.values(data.results).reduce((acc, result) => ({
        data: result.series
          ? acc.data.concat(result.series.map(serieObjectConstructor(result)))
          : acc.data.concat(tableObjectConstructor(result))
      }), { data: [] })
    ))
  }

  testDatasource() {
    return this.query({
      targets: [{
        rawSql: 'SELECT 1 FROM DUAL',
        format: 'time_series',
        refId: 'A'
      }],
      range: {
        from: Date.now() - 1000 * 60 * 60,
        to: Date.now()
      }
    }).then(() => ({ status: 'success', message: 'Database Connection OK' }))
  }
}

const serieObjectConstructor = options => serie => ({
  target: serie.name,
  datapoints: serie.points.map(([value, time]) => ([ Number(value), Number(time) ])),
  refId: options.refId,
  meta: options.meta
})

const tableObjectConstructor = query => (query.tables.map(t => ({
  type: 'table',
  refId: query.refId,
  meta: query.meta,
  columns: t.columns,
  rows: t.rows,
})))