import {MetricsPanelCtrl} from 'app/plugins/sdk';
import _ from 'lodash';
import './css/display-status-panel.css!';

const panelDefaults = {
  statusSettings: {
    fontSize: '60px',
    fontWeight: 'bold',
    textTransform: 'uppercase',
    color: '#C6D7D8'
  },
  elementSettings: {
    fontSize: '20px',
    fontWeight: 'normal',
    textTransform: 'normal',
    color: '#C6D7D8'
  },
  labelSettings: [
    "OK",
    "Scheduled downtime",
    "Acknowledged",
    "Flapping",
    "Warning",
    "Unknown",
    "Critical",
    "Unreachable",
    "Down"
  ],
  originalLabelSettings: [
    "OK",
    "Scheduled downtime",
    "Acknowledged",
    "Flapping",
    "Warning",
    "Unknown",
    "Critical",
    "Unreachable",
    "Down"
  ]

};

const log = msg => x => (console.log(msg, x), x)

export class DisplayStatus extends MetricsPanelCtrl {
  constructor($scope, $injector) {
    super($scope, $injector);
    _.defaultsDeep(this.panel, panelDefaults);

    this.events.on('data-received', this.onDataReceived.bind(this));
    this.events.on('init-edit-mode', this.onInitEditMode.bind(this));

    this.getColors().then(colors => this.allStates = colors);
  }

  onDataReceived(data=[]) {
    if (!this.allStates) {
      return setTimeout(this.onDataReceived.bind(this, data), 1000)
    }
    const value = pluck([0, 'datapoints', 0, 0], data)
    const state = this.allStates[value % this.allStates.length] || { label: 'n/A', color: '#000' }

    this.element = {
      name: pluck([0, 'target'], data) || 'Element not found'
    }

    this.status = {
      label: this.panel.labelSettings[value],
      color: state.color
    }

    this.render()
  }

  link(scope, elem, attrs, ctrl) {
    this.events.on('render', () => {
      elem.find('.panel-container').css('background-color', this.status.color)
      elem.find('.panel-title').css('color', '#FFF')
    })
  }

  getColors() {
    return (window.opmonColorStatus && window.opmonColorStatus.length > 0)
      ? Promise.resolve(window.opmonColorStatus)
      : fetch('/opmon/seagull/www/index.php/opinterface/action/retrieveStateColors', {
          credentials: 'same-origin',
          headers: {
            'X-Requested-With': 'XMLHttpRequest'
          }
        })
        .then(res => res.json())
        .then(colors => (window.opmonColorStatus = colors, colors))
  }

  onInitEditMode() {
    this.addEditorTab('Options', 'public/plugins/state/editor.html', 2);
  }
}

DisplayStatus.templateUrl = 'module.html';

const pluck = (keys, obj) => (
  keys.reduce((x, key) => x && x[key], obj)
)

const updateProperty = ctx => (scope, property, value) => {
  ctx[scope][property] = value
}

